@@ -2,25 +2,25 @@
 main app base class with context help for flow and app state changes
 ====================================================================
 
-The class :class:`HelpAppBase` provided by this namespace portion
-is extending your application with a context-sensitive help
-functionality.
+The class :class:`HelpAppBase` provided by this
+namespace portion is extending your application
+with a context-sensitive help functionality.
 
-The data-driven approach is providing a help system to your app
-which allows to add, edit and remove help texts without the need
-to change a single line of code. This gets achieved within
+The data-driven approach allows to add, edit and
+remove help texts without the need to change a
+single line of code. This gets achieved within
 :class:`HelpAppBase` by overriding the main app class methods
 :meth:`~ae.gui_app.MainAppBase.change_flow` and
 :meth:`~ae.gui_app.MainAppBase.change_app_state`.
 
-So for to add help support to the widgets of your app you only
-need to add/provide the help texts with a help id that is
-matching the value of the :attr:`ae_help_id` attribute of
-the widget you need help for.
+So for to add help support to the widgets of your app
+you only need to add/provide the help texts with a help id
+that is matching the value of the :attr:`ae_help_id`
+attribute of the widget you need help for.
 
-Additionally you can provide a separate i18n translation message
-file for each of the supported languages for to make your
-help texts multi-lingual.
+Additionally you can provide a separate i18n translation
+message file for each of the supported languages for to
+make your help texts multi-lingual.
 
 
 help layout implementation example
@@ -57,7 +57,7 @@ the attribute :attr:`~HelpAppBase.help_layout`::
 
     main_app.help_layout = HelpScreenContainerOrWindow()
 
-So :attr:`~HelpAppBase.help_layout is also used as a flag
+:attr:`~HelpAppBase.help_layout is also used as a flag
 of the help mode activity. By assigning `None` to this
 attribute the help mode will be deactivated::
 
@@ -101,11 +101,6 @@ using the prefix marker string defined by the module variable
 :data:`HELP_ID_PREFIX_FLOW` followed by
 the flow id of the flow widget.
 
-.. hint::
-    More information regarding the flow id you find in
-    the doc string of the module :mod:`ae.gui_app` in the
-    section :ref:`application flow`.
-
 For example the message id for a flow button with the
 flow action `'open'`, the object `'item'` and the (optional)
 flow key `'123456'` is resulting in the following help text
@@ -115,14 +110,21 @@ message id::
 
 If there is no need for a detailed message id that is taking
 the flow key into account, then simply create a help text
-message id without the flow key. The method
-:meth:`~HelpAppBase.help_display` does first search for
-a message id including the flow key in the available
-help text files and if not found it will automatically
-fallback to use a message id without the flow key::
+message id without the flow key.
+
+The method :meth:`~HelpAppBase.help_display` does first
+search for a message id including the flow key in the
+available help text files and if not found it will
+automatically fallback to use a message id without
+the flow key::
 
     `'help_flow#open_item'`
 
+.. hint::
+    More information regarding the flow id you find in
+    the doc string of the module :mod:`ae.gui_app` in the
+    section :ref:`application flow`.
+
 
 application state change context message id
 -------------------------------------------
@@ -156,11 +158,10 @@ like shown in the following example::
        ...
     }
 
-.. hint::
-    The provided `count` value can also be
-    included/displayed in the help text,
-    like shown in the `'zero'` count case of
-    the example.
+The provided `count` value can also be
+included/displayed in the help text,
+like shown in the `'zero'` count case of
+the example.
 
 
 pre- and post-change help texts
@@ -233,11 +234,12 @@ from ae.i18n import default_language, get_f_string, translation
 from ae.gui_app import FLOW_KEY_SEP, module_globals, id_of_flow, MainAppBase                # type: ignore
 
 
-__version__ = '0.1.7'
+__version__ = '0.1.8'
 
 
 HELP_ID_PREFIX_FLOW = 'help_flow#'
 HELP_ID_PREFIX_STATE = 'help_app_state#'
+
 IGNORED_HELP_FLOWS = (id_of_flow('close', 'flow_popup'), )          #: tuple of flow ids never search/show help text for
 IGNORED_HELP_STATES = ('flow_id', 'flow_path', 'win_rectangle')     #: tuple of app state names never searched help for
 
@@ -452,15 +454,15 @@ class HelpAppBase(MainAppBase, ABC):
         """
         return f'{HELP_ID_PREFIX_STATE}{state_name}'
 
-    def help_display(self, help_id: str, help_vars: Dict[str, Any], widget: Optional[Any] = None,
-                     key_suffix: str = '', must_have: bool = False) -> bool:
+    def help_display(self, help_id: str, help_vars: Dict[str, Any], key_suffix: str = '', must_have: bool = False
+                     ) -> bool:
         """ display help text to the user in activated help mode.
 
         :param help_id:         help id to show help text for.
         :param help_vars:       variables used in the conversion of the f-string expression to a string.
-                                items passed to the help text renderer:
+                                optional items passed to the help text renderer:
                                     * `count`: optional number used for to render a pluralized help text.
-        :param widget:          target widget for to show help text for.
+                                    * `self`: target widget for to show help text for.
         :param key_suffix:      suffix to the key used if the translation is a dict.
         :param must_have:       pass True for to display error help text and console output if no help text exists.
         :return:                True if help text got found and displayed.
@@ -485,26 +487,11 @@ class HelpAppBase(MainAppBase, ABC):
             help_id = short_help_id = has_trans['next_help_id']
             key_suffix = ''
 
-        wid = None
-        if help_id:     # if not in init/reset then ensure target widget and extend help_vars with call stack locals
-            wid = self.help_widget('ae_help_id', help_id, help_vars)
-            if not wid:
-                wid = self.widget_by_attribute('ae_help_id', help_id)
-                if wid:
-                    help_vars['self'] = wid
-                elif widget and widget not in (wid, self.help_activator):
-                    self.dpo(f"HelpAppBase.help_display(): target={wid}/{help_id} missing or IS NOT event_wid={widget}")
-        widget = wid or widget or self.help_activator
-
         hlw: Any = self.help_layout
-        hlw.target = widget
+        hlw.target = self.help_widget(help_id, help_vars)
         self.change_observable('displayed_help_id', help_id)
 
-        glo_vars = module_globals.copy()
-        glo_vars.update(globals())
-        glo_vars['_add_base_globals'] = True
-        help_vars['app'] = self.framework_app
-        help_vars['main_app'] = self
+        glo_vars, help_vars = self.help_variables(help_vars)
         hlw.help_text = get_f_string(short_help_id, key_suffix=key_suffix, glo_vars=glo_vars, loc_vars=help_vars)
 
         self.ensure_top_most_z_index(hlw)
@@ -549,24 +536,49 @@ class HelpAppBase(MainAppBase, ABC):
             return hid[len(HELP_ID_PREFIX_FLOW):]
         return ''
 
-    def help_widget(self, attr_name: str, attr_value: str, help_vars: Dict[str, Any]) -> Optional[Any]:
-        """ find help target widget via attribute name/value and extend :paramref:`~help_widget.help_vars`.
+    def help_variables(self, help_vars: Dict[str, Any]) -> Tuple[Dict[str, Any], Dict[str, Any]]:
+        """ determine globals and extend help vars for help text evaluation/translation
 
-        :param attr_name:       widget attribute name for to detect widget and call stack locals.
-        :param attr_value:      widget attribute value for to detect widget and call stack locals
-        :param help_vars:       help env variables, to be extended with event activation stack frame locals.
-        :return:                found help target widget or None if not found.
+        :param help_vars:       initial help variable locals that will be extended and returned.
+        :return:                global and local variable dicts.
         """
-        depth = 3
-        while depth <= 15:
-            # look for FlowButton widget in kv/enaml rule call stack frame for translation text context
-            _gfv, lfv, _deep = stack_vars('do not skip ae_gui_app', min_depth=depth, max_depth=depth)
-            widget = lfv.get('self')
-            if getattr(widget, attr_name, None) == attr_value:
-                help_vars.update(lfv)
-                return widget
-            depth += 1
+        glo_vars = module_globals.copy()
+        glo_vars.update(globals())
+        glo_vars['_add_base_globals'] = True        # instruct ae.inspector.try_eval to add ae.core globals
 
-        self.dpo(f"help_widget(): {attr_name} value '{attr_value}' not found up to the {depth - 5}th stack level")
+        help_vars['app'] = self.framework_app
+        help_vars['main_app'] = self
 
-        return None
+        return glo_vars, help_vars
+
+    def help_widget(self, help_id: str, help_vars: Dict[str, Any]) -> Any:
+        """ ensure/find help target widget via attribute name/value and extend :paramref:`~help_widget.help_vars`.
+
+        :param help_id:         widget.ae_help_id attribute value for to detect widget and call stack locals.
+        :param help_vars:       help env variables, to be extended with event activation stack frame locals
+                                and a 'self' key with the help target widget.
+        :return:                found help target widget or self.help_activator if not found.
+        """
+        wid = help_vars.get('self')
+        if not wid:
+            if help_id:
+                # first look for FlowButton widget in kv/enaml rule call stack frame for translation text context
+                depth = 3
+                while depth <= 15:
+                    _gfv, lfv, _deep = stack_vars("", min_depth=depth, max_depth=depth)  # "" to not skip ae.kivy_help
+                    widget = lfv.get('self')
+                    if getattr(widget, 'ae_help_id', None) == help_id:
+                        help_vars.update(lfv)
+                        return widget
+                    depth += 1
+
+                # then search the widget tree
+                wid = self.widget_by_attribute('ae_help_id', help_id)
+                if not wid:
+                    self.vpo(f"HelpAppBase.help_widget(): widget with ae_help_id '{help_id}' not found")
+
+            if not wid:
+                wid = self.help_activator
+            help_vars['self'] = wid
+
+        return wid
